// We will need the CSPD library functions
#include "CSPDLib.H"

// We also need standard and console IO functions
#include <StdIO.H>
#include <ConIO.H>
#include <String.H>

// LCD_Line refers to the line number of the cursor on the LCD.  This are used
// by the LCD routines to manage the cursor
BYTE												LCD_Line, LCD_Col;
// The LCD driver gives us access to the LCD through stdprn which we access
// this rather simple alias
FILE*												LCD;

// Tell the LCD driver that the next byte is a command byte
#define LCD_CMD								160
// Clear the LCD and put the cursor at home
#define LCD_CLEARHOME						0x01
// Set the IO properties of the LCD correctly (THIS IS BOARD SPECIFIC to the
// Flashlite186 other boards may differ)
#define LCD_IOSETUP							0x40
// Increment data pointer, no display shift
#define LCD_CHARENTRY						0x06
// Turn on the display without the cursor
#define LCD_DISPLAY							0x0C
// This is a simple macro to help set the cursor position
#define LCD_SETPOS(x)						(0x80 | x)
// These values are to be used with the LCD_SETPOS macro
BYTE												LineStart[4] = { 0x00, 0x40, 0x14, 0x54 };

// Force the cursor to move to the position we think it should be in
void												LCD_ForcePos(void)
{
	// Send a LCD_CMD to tell the driver the next byte is a command
	// Then send a SETPOS command
	fprintf(LCD, "%c%c",LCD_CMD,LCD_SETPOS(LineStart[LCD_Line] + LCD_Col));
}

// Clear the LCD and move the cursor to home
void												LCD_ClearHome(void)
{
	// Record the new cursor position
	LCD_Line = 0;
	LCD_Col = 0;
	// Send a LCD_CMD to tell the driver the next byte is a command
	// Then send a CLEARHOME command
	fprintf(LCD,"%c%c",LCD_CMD,LCD_CLEARHOME);
}

// Effectively print a newline character (move the cursor to a newline)
void												LCD_NewLine(void)
{
	// Move to the next line
	LCD_Line++;
	LCD_Col = 0;
	// If we moved off the last line of the display then CLEARHOME it
	if (LCD_Line == 4) LCD_ClearHome();
	else
	{
		// Force the cursor to actually go there
		LCD_ForcePos();
	}
}

// Initialize the LCD
void												LCD_Init(void)
{
	// The LCD driver makes the LCD availible on the standard printer port
	LCD = stdprn;
	// We'll start on line 0
	LCD_Line = 0;
	LCD_Col = 0;
	// Make sure that the Borland C++ 4.52 standard libraries don't try to buffer
	// the serial port characters
	setbuf(LCD, NULL);
	// Perform some initialization on the LCD
	fprintf(LCD,"%c%c",LCD_CMD,LCD_IOSETUP);
	fprintf(LCD,"%c%c",LCD_CMD,LCD_DISPLAY);
	fprintf(LCD,"%c%c",LCD_CMD,LCD_CHARENTRY);
	// Clear the display and move the cursor to home
	LCD_ClearHome();
}


// This will send a command string to the GPS
WORD												GPS_SendCommand(char* CommandString)
{
	unsigned long		i, j;

	i = 0xFFFFl;
	// We don't return until the GPS gives us a return code, ACK or NAK
	while (CSPD_GetChar(0) == 0xFFFF)
	{
		// If a key was hit
		if (kbhit())
		{
			// Wait a bit to ensure the serial ports arent fouling up the debug port
			for (j = 0; j < 0xFFFF; j++);
			// Now make sure a key was hit and if it was return failure
			if (kbhit()) if (getch() == 0x1B) return 1;
			// Make it so that we dump the command out again
			i += 0xFFFF;
		}
		// If we just started or have been waiting a long time for the command
		// response send it again
		if (i >= 0xFFFFl)
		{
			// Actually send the command
			CSPD_PutBlock(0, CommandString, strlen(CommandString));
			// Reset the resend counter
			i = 0;
		}
		// Increment this counter so we know when to resend the command
		i++;
	}
	// Clear the response out of the buffer
	while (CSPD_GetChar(0) != 0xFFFF);

	// Return success
	return 0;
}

#define STRING_LEN	50
#define STRING_MAX	48

void main(/*int argc*/)
{
	// We'll use these buffers for storing the GPS information
	char					Time[STRING_LEN], LastTime[STRING_LEN] = "";
	char					Lat[STRING_LEN], Long[STRING_LEN], Sats[STRING_LEN];
	char					COG[STRING_LEN], SOG[STRING_LEN], LatDir, LongDir;
	char					DirChar0[] = "NNESSSWN", DirChar1[] = " E E W W";
	// This is the GPS status message prefix
	char					GGAMessage[] = "$GPGGA,", VTGMessage[] = "$GPVTG,";
	// We'll use i to insert some waits
	unsigned long		i;
	// State is the state of the GPS status decoder
	// Hour Minute Second and Degree are used to format the output to the LCD
	WORD					State = 0, Char, Count, Hour, Minute, Degree, Second;
	WORD					COGNum, SOGNum, GoodData;
	unsigned long		ULong;

	// Initialize the LCD Note that before running this example you'll need to
	// load the LCD_186 driver
	LCD_Init();

	// The GPS board will want to talk at 4800 baud on the TTL level lines that
	// serial port 0 provides
	CSPD_SetBaud(0, 4800);
	// The GPS board does not need or want any handshaking
	CSPD_RxSetHandshaking(0, 0);
	CSPD_TxSetHandshaking(0, 0);
	// Clear out the serial port buffers so that garbage characters produced
	// before this program was run are ignored
	CSPD_TxClearBuffer(0);
	CSPD_RxClearBuffer(0);

	// Clear the GPS memory if anything was on the command line
	/*if (argc > 1)
	{
		if (GPS_SendCommand("$PASHS,INI,4,4,,,1,\r\n")) return;
	}
	else
	{*/
		// Reset the GPS (don't clear the memory)
		if (GPS_SendCommand("$PASHS,INI,4,4,,,0,\r\n")) return;
	//}

	// Enable the speed and position messages
	if (GPS_SendCommand("$PASHS,NME,VTG,A,ON\r\n")) return;
	if (GPS_SendCommand("$PASHS,NME,GGA,A,ON\r\n")) return;

	// Loop until the user hits a key
	while(1)
	{
		//printf("State: %u\n", State);
		if (kbhit())
		{
			for (i = 0; i < 0xFFFF; i++);
			if (kbhit()) { if (getch() == 0x1B) return; }
		}
		// Try to read a character from the serial port, if there wasn't one then
		// just loop until there is
		if ((Char = CSPD_GetChar(0)) == 0xFFFF) continue;
		// We got a character so now we have to decode it
		if (State < 7)
		{
			// States 0-6 correspond to characters 0-6 in the $GPGGA, status message
			// header
			if ((char)Char == GGAMessage[State])
			{
				// We got the right character so expect the next one
				State++;
				// If we moved past the $GPGGA, header we need to zero Count for the
				// next part of the state machine
				if (State == 7) Count = 0;
			}
			// if one of them gets lost or is wrong then just wait for the next
			// $GPGGA, message
			else State = 0;
		}
		else if (State == 7) // We are getting the UTC (TIME) part of the message
		{
			// Check if we got the ',' indicating the end of the time field
			if (Char == ',')
			{
				// Terminate the time string properly
				Time[Count] = '\0';
				// Set the count for the next state machine state
				Count = 0;
				// Go to the next state machine state
				State++;
			}
			else
			{
				// Store the character we got
				if (Count <= STRING_MAX) Time[Count] = Char;
				// Get ready to store the next character
				Count++;
			}
		}
		else if (State == 8) // We're getting the latitude (LAT) information
		{
			// Check if we got the ',' indicating the end of the latitude field
			if (Char == ',')
			{
				// Terminate the latitude string properly
				Lat[Count] = '\0';
				// Set the count for the next state machine state
				Count = 0;
				// Go to the next state machine state
				State++;
			}
			else
			{
				// Store the character we got
				if (Count <= STRING_MAX) Lat[Count] = Char;
				// Get ready to store the next character
				Count++;
			}
		}
		else if (State == 9) // We're getting the latitude direction ('N' or 'S')
		{
			// Check if we got the ',' indicating the end of the field
			if (Char == ',') State++;
			else LatDir = Char;
		}
		else if (State == 10) // LONG
		{
			// Check if we got the ',' indicating the end of the longitude field
			if (Char == ',')
			{
				// Terminate the longitude string properly
				Long[Count] = '\0';
				// Set the count for the next state machine state
				Count = 0;
				// Go to the next state machine state
				State++;
			}
			else
			{
				// Store the character we got
				if (Count <= STRING_MAX) Long[Count] = Char;
				// Get ready to store the next character
				Count++;
			}
		}
		else if (State == 11) // We're getting the longitude direction ('E' or 'W')
		{
			// Check if we got the ',' indicating the end of the field
			if (Char == ',') State++;
			else LongDir = Char;
		}
		else if (State == 12) // We're getting the position type...
		{
			// Check if we got the ',' indicating the end of the field
			if (Char == ',') State++;
		}
		else if (State == 13) // Getting the number of active satellites...
		{
			// Check if we got the ',' indicating the end of the field
			if (Char == ',')
			{
				// If we didn't actually get any characters between the commas
				// then store a space rather than a '\0' so that we can still print
				// the actual latitude, which is right after this one character
				Sats[Count] = '\0';
				// Set the count for the next state machine state
				Count = 0;
				// Reset the state machine for the next status message
				State++;
			}
			else
			{
				// Store the character we got
				if (Count <= STRING_MAX) Sats[Count] = Char;
				// Get ready to store the next character
				Count++;
			}
		}
		else if ((State >= 14) && (State <= 20)) // Getting the "$GPVTG," header
		{
			// States 14-20 correspond to characters 0-6 in the $GPVTG, status message
			// header
			if ((char)Char == VTGMessage[State - 14])
			{
				// We got the right character so expect the next one
				State++;
				// If we moved past the $GPVTG, header we need to zero Count for the
				// next part of the state machine
				if (State == 21) Count = 0;
			}
			// if one of them gets lost or is wrong then just wait for the next
			// $GPGGA, message
			else if (State > 14) State = 0;
		}
		else if (State == 21) // Course Over Ground
		{
			// Check if we got the ',' indicating the end of the COG field
			if (Char == ',')
			{
				// Terminate the COG string properly
				COG[Count] = '\0';
				// Set the count for the next state machine state
				Count = 0;
				// Go to the next state machine state
				State++;
			}
			else
			{
				// Store the character we got
				if (Count <= STRING_MAX) COG[Count] = Char;
				// Get ready to store the next character
				Count++;
			}
		}
		else if ((State >= 22) && (State <= 26)) // We're getting the position type...
		{
			// Check if we got the ',' indicating the end of the field
			if (Char == ',') State++;
		}
		else if (State == 27) // We're getting speed over ground
		{
			if (Char == ',')
			{
				// Terminate the SOG string properly
				SOG[Count] = '\0';
				// Set the count for the next state machine state
				Count = 0;
				// Go to the next state machine state
				State = 0;

				// Make sure we don't display garbage data from the GPS (it
				// returns garbage when it loses satellite lock or has just been
				// turned on)  We know the data is garbage when the time is the same
				// as the last time returned
				GoodData = (strcmp(Time, LastTime) != 0) && (LastTime[0] != '\0');

				// Wait before we fiddle with IO stuff so that the serial port
				// can settle down
				for (i = 0; i < 0xFF; i++);
				// Move the LCD cursor to home
				LCD_Col = 0;
				LCD_Line = 0;
            LCD_ForcePos();
				// Do another wait while the LCD thinks about things
				for (i = 0; i < 0xFF; i++);
				// Print the time, latitude and longitude to the LCD and the console

				// Print the latitude nicely to the LCD and console.
				if (GoodData && (sscanf(strchr(Lat, '.') - 2, "%2u.%5lu", &Minute, &ULong) == 2))
				{
					ULong *= 6;
					Second = (WORD)(ULong / 10000);
					//printf("%lu %u ", ULong, Second);
					ULong -= ((unsigned long)Second) * 10000l;
					//printf("%lu\n", ULong);
					*(strchr(Lat, '.') - 2) = '\0';
					if (sscanf(Lat, "%3u", &Degree) == 1)
					{
						fprintf(LCD, "Lat  %3.3u:%02.2u:%02.2u.%04.4lu%c", Degree, Minute, Second, ULong, LatDir);
						printf("Lat  %3.3u:%02.2u:%02.2u.%04.4lu%c\n", Degree, Minute, Second, ULong, LatDir);
					}
					else
					{
						fprintf(LCD, "Lat  ---:--:--.-----");
						printf("Lat  ---:--:--.-----\n");
					}
				}
				else
				{
					fprintf(LCD, "Lat  ---:--:--.-----");
					printf("Lat  ---:--:--.-----\n");
				}
				LCD_NewLine();
				Lat[0] = '\0';
				Lat[1] = '\0';
				LatDir = '\0';

				// Print the longitude nicely to the LCD console.
				if (GoodData && (sscanf(strchr(Long, '.') - 2, "%2u.%5lu", &Minute, &ULong) == 2))
				{
					ULong *= 6;
					Second = (WORD)(ULong / 10000);
					ULong -= ((unsigned long)Second) * 10000l;
					*(strchr(Long, '.') - 2) = '\0';
					if (sscanf(Long, "%3u", &Degree) == 1)
					{
						fprintf(LCD, "Long %3.3u:%02.2u:%02.2u.%04.4lu%c", Degree, Minute, Second, ULong, LongDir);
						printf("Long %3.3u:%02.2u:%02.2u.%04.4lu%c\n", Degree, Minute, Second, ULong, LongDir);
					}
					else
					{
						fprintf(LCD, "Long ---:--:--.-----");
						printf("Long ---:--:--.-----\n");
					}
				}
				else
				{
					fprintf(LCD, "Long ---:--:--.-----");
					printf("Long ---:--:--.-----\n");
				}
				LCD_NewLine();
				Long[0] = '\0';
				Long[1] = '\0';
				LongDir = '\0';

				// Print the time and active satellite count nicely
				if (GoodData && (sscanf(Time, "%2u%2u%2u", &Hour, &Minute, &Second) == 3) && (sscanf(Sats, "%lu", &ULong) == 1))
				{
					fprintf(LCD, "GMT %02.2u:%02.2u:%02.2u Sats:%02lu", Hour, Minute, Second, ULong);
					printf("GMT %02.2u:%02.2u:%02.2u Sats:%02lu\n", Hour, Minute, Second, ULong);
				}
				else
				{
					fprintf(LCD, "GMT --:--:-- Sats:--");
					printf("GMT --:--:-- Sats:--\n");
				}
				strcpy(LastTime, Time);
				Time[0] = '\0';
				Time[1] = '\0';
				Sats[0] = '\0';
				Sats[1] = '\0';
				LCD_NewLine();

				// Print the speed and course over ground
				if (GoodData && (sscanf(COG, "%3u", &COGNum) == 1) && (sscanf(SOG, "%3u.%2lu", &SOGNum, &ULong) == 2))
				{
					// If the speed is tiny assume its really 0 and the GPS was just
					// wrong because of random changes in computed position
					if ((SOGNum == 0) && (ULong <= 1))
					{
						fprintf(LCD, "Speed   0.00kmph    ");
						printf("Speed   0.00kmph\n");
					}
					else
					{
						// Print the speed and course over ground
						COGNum = (((COGNum << 1) + 45) / 90) % 8;
						fprintf(LCD, "Speed %3u.%02lukmph %c%c ", SOGNum, ULong, DirChar0[COGNum], DirChar1[COGNum]);
						printf("Speed %3u.%02lukmph %c%c\n", SOGNum, ULong, DirChar0[COGNum], DirChar1[COGNum]);
					}
				}
				else
				{
					fprintf(LCD, "Speed ---.--kmph -- ");
					printf("Speed ---.--kmph --\n");
				}
				COG[0] = '\0';
				COG[1] = '\0';
				SOG[0] = '\0';
				SOG[1] = '\0';
			}
			else
			{
				// Store the character we got
				if (Count <= STRING_MAX) SOG[Count] = Char;
				// Get ready to store the next character
				Count++;
			}
		}
	}
}